
/*

	+++ Example for using ExpDClib +++

	see 'Draw'-function for routines used to draw on DC

	to export the content of a DC you can use a DC-Handle and call lib-functions (ExpDCStartPage, ExpDCEndPage...)
	or define '_USE_EXPCLASS_' to use the lib with a class (CDC)

	if you want use exceptions, define '_USE_EXPEXCEPTIONS_'

*/

#include "stdafx.h"
#include "ExpSample.h"

#define _USE_EXPCLASS_
#define _USE_EXPEXCEPTIONS_
#include "ChildView.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define SCROLLD 5 //mm

#define DATADIR	_T("..\\..\\data\\")

#ifdef _UNICODE
#pragma comment(linker,"/entry:\"wWinMainCRTStartup\"")
#endif

// Add the lib in Projekt->Settings->Linker or use this line:
#pragma comment(linker, "/defaultlib:\"..\\..\\..\\out\\bin\\expdc.lib\"")



BEGIN_MESSAGE_MAP(CChildView,CWnd)
	//{{AFX_MSG_MAP(CChildView)
	ON_WM_CREATE()
	ON_WM_SIZE()
	ON_WM_PAINT()
	ON_WM_VSCROLL()
	ON_WM_HSCROLL()
	ON_WM_ERASEBKGND()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()



CChildView::CChildView()
: m_pages(2), m_zoom(100), m_scrd(0, 0), m_offs(0, 0)
{
	char d[MAX_PATH];
	GetModuleFileNameA(::AfxGetInstanceHandle(), d, MAX_PATH);
	long l = strlen(d);
	while(l-- > 0)
	{
		if(d[l] == '\\')
		{
			d[l] = 0;
			break;
		}
	}
	SetCurrentDirectoryA(d);

	m_szPage.cx = 2100;
	m_szPage.cy = 2980;
	m_szPageDP = m_szPage;
	m_mapmode = MM_LOMETRIC;
	m_expfile = _T("example.pdf");

	ExpDCGetPDFOptions(&m_pdfopts);
}

CChildView::~CChildView()
{
}

void CChildView::Export()
{
	ExpDCUseExceptions(true);


	ExpDCSetPDFOptions(m_pdfopts);
	time_t t;
	time(&t);
	ExpDCSetPDFInfo(localtime(&t), "tb", "ExpSample", "Example PDF", "Using ExpDClib", "ExpDCLib PDF DC");

	// change FontSubstitution, in this example "Times New Roman" will be replaced with core-font
	ExpDCSetFontSubstitution(cf_times, "Times New Roman");

	CRect r(0, 0, m_szPage.cx, m_szPage.cy);
	long szd;
	const char* data = NULL;

	try
	{

#ifdef _USE_EXPCLASS_

		CExpDC expdc(r, NULL);
		Print(&expdc, 0, 1);

#else
		HDC hExpDC = ExpDCCreate(r, NULL);
		Print(hExpDC, 0, 1, true);

#endif

		data = ExpDCGetData(&szd);
		if((data != NULL) && (m_expfile != _T("")))
		{
			FILE* file = _tfopen(m_expfile, _T("wb"));
			if(file != NULL)
			{
				fwrite(data, sizeof(char), szd, file);
				fclose(file);
			}
			else
			{
				CString s;
				s.Format(_T("Error open file '%s'"), m_expfile);
				MessageBox(s);
			}
		}
	}
	catch(CExpDCException e)
	{
		CString s;
		s.Format(_T("Error: %d (%S)"), e.GetError(), e.GetErrInfo());
		MessageBox(s);
	}

// if class is not used, ExpDCRelease must be called!
#ifndef _USE_EXPCLASS_
	ExpDCRelease();
#endif

}

#ifndef _USE_EXPCLASS_

void CChildView::Print(HDC hDC, long pgfrom, long pgto, bool bExp)
{
	DOCINFO di;
	memset(&di, 0, sizeof(DOCINFO));
	di.cbSize = sizeof(DOCINFO);
	di.lpszDocName = _T("Export-DC Sample Document");

	if(bExp)
		ExpDCStartDoc();
	else
		StartDoc(hDC, &di);

	// in this example, "Draw" has CDC* as parameter
	CDC dc;
	dc.Attach(hDC);

	for(int i = pgfrom - 1; i <= pgto - 1; i++)
	{
		if(bExp)
			ExpDCStartPage();
		else
			dc.StartPage();

		int mm = dc.SetMapMode(m_mapmode);
		Draw(&dc, i);
		dc.SetMapMode(mm);

		if(bExp)
			ExpDCEndPage();
		else
			dc.EndPage();
	}

	if(bExp)
		ExpDCEndDoc();
	else
		EndDoc(hDC);

	dc.Detach();

}

#endif

void CChildView::OnPrint()
{
	CPrintDialog dlg(FALSE, PD_RETURNDC);
	PRINTDLG* pPD = &dlg.m_pd;
	pPD->nMinPage = 1;
	pPD->nMaxPage = m_pages;
	pPD->nFromPage = 1;
	pPD->nToPage = m_pages;

	if(dlg.DoModal() != IDOK)
		return;

	if(pPD->hDC)
	{
#ifdef _USE_EXPCLASS_
		CDC dc;
		dc.Attach(pPD->hDC);

		Print(&dc, pPD->nFromPage - 1, pPD->nToPage - 1);

		dc.Detach();
#else
		Print(pPD->hDC, false, pPD->nFromPage - 1, pPD->nToPage - 1);
#endif

		DeleteDC(pPD->hDC);
	}
}

BOOL CChildView::OnCreate(LPCREATESTRUCT cs) 
{
	BOOL res = CWnd::OnCreate(cs);
	if(res == 0)
	{
		ShowScrollBar(SB_BOTH);
		CPaintDC dc(this);
		dc.SetMapMode(m_mapmode);
		m_szPageDP = m_szPage;
		dc.LPtoDP(&m_szPageDP);
		m_scrd.cx = SCROLLD * 10; // MM_LOMETRIC
		m_scrd.cy = SCROLLD * 10;
		dc.LPtoDP(&m_scrd);
	}
	return res;
}


BOOL CChildView::PreCreateWindow(CREATESTRUCT& cs) 
{
	if (!CWnd::PreCreateWindow(cs))
		return FALSE;

	cs.dwExStyle |= WS_EX_CLIENTEDGE;
	cs.style &= ~WS_BORDER;
	cs.lpszClass = AfxRegisterWndClass(CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS, ::LoadCursor(NULL, IDC_ARROW), HBRUSH(COLOR_WINDOW + 1), NULL);

	return TRUE;
}

BOOL CChildView::OnEraseBkgnd(CDC* pDC)
{
	CBrush brhg;
	brhg.CreateSolidBrush(RGB(100, 100, 100));

	int sd = pDC->SaveDC();
	CRect r, cr;
	GetClientRect(r);
	cr.left = m_offs.cx;
	cr.right = cr.left + (long)(m_szPageDP.cx * m_zoom / 100.0);
	cr.top = m_offs.cy;
	cr.bottom = cr.top + (long)(m_szPageDP.cy * m_zoom / 100.0);

	pDC->ExcludeClipRect(cr);
	pDC->FillRect(r, &brhg);
	if(brhg.m_hObject)
		brhg.DeleteObject();
	pDC->RestoreDC(sd);
	return TRUE;
}

void CChildView::OnPaint() 
{
	CPaintDC dc(this);

	int mm = dc.SetMapMode(m_mapmode);
	CRect cr;
	GetClientRect(cr);

	CSize szv(cr.Width(), cr.Height());
	dc.LPtoDP(&szv);
	szv.cx = (long)(szv.cx * m_zoom / 100.0);
	szv.cy = -(long)(szv.cy * m_zoom / 100.0);

	int sbm = dc.SetStretchBltMode(HALFTONE);
	mm = dc.SetMapMode(MM_ANISOTROPIC);
	CSize offs = m_offs;

	CBrush brhg;
	brhg.CreateSolidBrush(RGB(255, 255, 255));

	for(int i = 0; i < m_pages; i++)
	{
		dc.SetViewportOrg(offs.cx, offs.cy);
		dc.SetWindowExt(cr.Width(), cr.Height());
		dc.SetViewportExt(szv);

		CRect r(0, 0, m_szPage.cx, -m_szPage.cy);
		dc.FillRect(r, &brhg);
		Draw(&dc, i);

		offs.cy += (m_szPageDP.cy * m_zoom / 100) + 10;
	}

	brhg.DeleteObject();

	dc.SetMapMode(mm);
	dc.SetStretchBltMode(sbm);
}

void CChildView::Draw(CDC* pDC, long page)
{
	pDC->SetTextColor(RGB(0, 0, 0));
	pDC->SetBkColor(RGB(255, 255, 255));
	pDC->SetBkMode(TRANSPARENT);

	LOGFONT lf;
	memset(&lf, 0, sizeof(LOGFONT));
	_tcscpy(lf.lfFaceName, _T("Arial"));
	CFont fnt;
	CFont* fold = NULL;

	LOGBRUSH lb;
	CBrush brsh, *bold = NULL, *bprev = NULL;
	CBrush brshbm1, brshbm2, brshbm3, brshbm4, brshbm5, brshbm6;

	CPen pen;
	CPen* pold = NULL;
	SelectPen(pDC, PS_SOLID, 5, RGB(0, 0, 250), &pen, &pold);

	CString s;

	lf.lfHeight = 20;
	fold = SelectFont(pDC, &lf, &fnt, NULL);
	s.Format(_T("Page %d"), page + 1);
	pDC->TextOut(100, -50, s);

	if(page == 0)
	{
		pDC->MoveTo(100, -230);
		pDC->LineTo(m_szPage.cx - 100, -230);


		SelectBrush(pDC, NULL, &brshbm1, &bold, _T("ptrn1-8x8.bmp"));
		SelectBrush(NULL, NULL, &brshbm2, &bold, _T("ptrn2-8x8.bmp"));
		SelectBrush(NULL, NULL, &brshbm3, &bold, _T("ptrn1-32x32.bmp"));
		SelectBrush(NULL, NULL, &brshbm4, &bold, _T("bg1.bmp"));
		SelectBrush(NULL, NULL, &brshbm5, &bold, _T("ptrn2-32x32.bmp"));
		SelectBrush(NULL, NULL, &brshbm6, &bold, _T("ptrn3-32x32.bmp"));

		long y = -400;
		int i = 0;

		if(SelectBrush(pDC, NULL, &brsh, &bold, _T("ptrn1-8x8.bmp")))
		{
			memset(&lf, 0, sizeof(LOGFONT));
			lf.lfHeight = 130;
			lf.lfWeight = FW_BOLD;
			SelectFont(pDC, &lf, &fnt, fold);
			pDC->SetBkMode(TRANSPARENT);
			pDC->BeginPath();
			pDC->TextOut(400, -80, "Simply export DC to PDF");
			pDC->EndPath();
			pDC->FillPath();
		}

		lf.lfHeight = 50;
		lf.lfWeight = FW_NORMAL;
		_tcscpy(lf.lfFaceName, _T("Arial"));
		SelectFont(pDC, &lf, &fnt, fold);
		pDC->TextOut(100, y, "Use gdi-functions to fill the DC as usual, replace the (Printer-)DC with ExportDC to create PDF.");

		y -= 300;
		for(i = 0; i < 3; i++)
		{
			memset(&lf, 0, sizeof(LOGFONT));
			lf.lfHeight = 30;

			switch(i)
			{
			case 0:	_tcscpy(lf.lfFaceName, _T("Arial")); break;
			case 1: _tcscpy(lf.lfFaceName, _T("Courier New")); break;
			case 2: _tcscpy(lf.lfFaceName, _T("Comic Sans MS")); break;
			}

			SelectFont(pDC, &lf, &fnt, fold);

			pDC->GetTextFace(s);
			pDC->TextOut(100, y + -(i * 50), s);

			lf.lfWeight = FW_BOLD;
			SelectFont(pDC, &lf, &fnt, fold);
			pDC->TextOut(300, y + -(i * 50), "Bold");

			lf.lfWeight = 0;
			lf.lfItalic = 1;
			SelectFont(pDC, &lf, &fnt, fold);
			pDC->TextOut(400, y + -(i * 50), "Italic");

			lf.lfWeight = FW_BOLD;
			lf.lfItalic = 1;
			SelectFont(pDC, &lf, &fnt, fold);
			pDC->TextOut(500, y + -(i * 50), "BoldItalic");

			lf.lfStrikeOut = 1;
			lf.lfUnderline = 1;
			SelectFont(pDC, &lf, &fnt, fold);
			pDC->TextOut(650, y + -(i * 50), "strikeout and underline");
		}

		y -= 300;
		memset(&lf, 0, sizeof(LOGFONT));
		lf.lfHeight = 40;
		_tcscpy(lf.lfFaceName, _T("Arial Unicode MS"));
		SelectFont(pDC, &lf, &fnt, fold);
		USHORT* t = L"\x20AC  \x272B  \x33B2  \x2776  \x2777  \x2778";
		TextOutW(pDC->m_hDC, 100, y, t, wcslen(t));
		SelectPen(pDC, PS_SOLID, 1, RGB(150, 150, 150), &pen, &pold);

		y -= 250;
		lb.lbStyle = BS_PATTERN;

		pDC->SelectObject(&brshbm3);
		pDC->RoundRect(100, y, 250, y - 150, 50, 50);

		pDC->SelectObject(&brshbm2);
		pDC->RoundRect(210, y - 40, 290, y - 110, 20, 20);

		pDC->SelectObject(&brshbm1);
		pDC->RoundRect(145, y - 130, 190, y - 170, 20, 20);

		pDC->SetBkMode(OPAQUE);
		pDC->SetBkColor(RGB(128, 250, 200));
		lb.lbColor = RGB(50, 60, 10);
		lb.lbStyle = BS_HATCHED;
		lb.lbHatch = HS_DIAGCROSS;

		SelectBrush(pDC, &lb, &brsh, &bold);
		pDC->Ellipse(400, y, 600, y - 100);

		lb.lbHatch = HS_VERTICAL;
		SelectBrush(pDC, &lb, &brsh, &bold);
		pDC->Ellipse(400, y - 110, 600, y - 210);

		lb.lbHatch = HS_CROSS;
		SelectBrush(pDC, &lb, &brsh, &bold);
		pDC->Ellipse(620, y, 820, y - 100);

		lb.lbHatch = HS_BDIAGONAL;
		SelectBrush(pDC, &lb, &brsh, &bold);
		pDC->Ellipse(620, y - 110, 820, y - 210);

		lb.lbHatch = HS_FDIAGONAL;
		SelectBrush(pDC, &lb, &brsh, &bold);
		pDC->Ellipse(840, y, 1040, y - 100);

		lb.lbHatch = HS_HORIZONTAL;
		SelectBrush(pDC, &lb, &brsh, &bold);
		pDC->Ellipse(840, y - 110, 1040, y - 210);

		pDC->SetBkColor(RGB(255, 0, 0));
		lb.lbStyle = BS_HATCHED;
		lb.lbHatch = HS_DIAGCROSS;
		SelectPen(pDC, PS_GEOMETRIC | PS_DASH, 10, RGB(0, 0, 255), &pen, &pold, &lb);

		y -= 50;
		pDC->MoveTo(1100, y);
		pDC->LineTo(1600, y);

		lb.lbHatch = HS_DIAGCROSS;
		SelectPen(pDC, PS_GEOMETRIC | PS_DASHDOTDOT, 10, RGB(0, 0, 255), &pen, &pold, &lb);

		pDC->MoveTo(1100, y - 30);
		pDC->LineTo(1600, y - 30);

		lb.lbStyle = BS_PATTERN;
		SelectPen(pDC, PS_GEOMETRIC | PS_DASHDOTDOT, 30, 0, &pen, &pold, &lb, _T("ptrn2-8x8.bmp"));

		pDC->MoveTo(1100, y - 80);
		pDC->LineTo(1600, y - 80);

		SelectPen(pDC, PS_GEOMETRIC | PS_DASHDOTDOT, 30, 0, &pen, &pold, &lb, _T("ptrn1-32x32.bmp"));

		pDC->MoveTo(1100, y - 130);
		pDC->LineTo(1600, y - 130);

		y -= 600;
		long d = 600;
		long d05 = d / 2;
		SelectPen(pDC, PS_SOLID, 2, RGB(0, 0, 50), &pen, &pold);
		pDC->SelectObject(&brshbm4);
		pDC->BeginPath();
		pDC->SetArcDirection(AD_COUNTERCLOCKWISE);
		
		// ArcTo in Path not working on Win9x
		pDC->MoveTo(400, y);
		pDC->LineTo(100, y);
		pDC->ArcTo(400 - d05, y + d05, 400 + d05, y - d05, 400 - d05, y, 400 - 200, y - 100);
		pDC->LineTo(400, y);
		pDC->EndPath();
		pDC->StrokeAndFillPath();
		
		pDC->SelectObject(&brshbm5);
		pDC->BeginPath();
		pDC->ArcTo(400 - d05, y + d05, 400 + d05, y - d05, 400 - 200, y - 100, 400, y - 400);
		pDC->LineTo(400, y);
		pDC->EndPath();
		pDC->StrokeAndFillPath();

		pDC->SelectObject(&brshbm6);
		pDC->BeginPath();
		pDC->ArcTo(400 - d05, y + d05, 400 + d05, y - d05, 400, y - 400, 700, y - 400);
		pDC->LineTo(400, y);
		pDC->EndPath();
		pDC->StrokeAndFillPath();

		lb.lbStyle = BS_SOLID;
		lb.lbColor = RGB(200, 200, 200);
		SelectBrush(pDC, &lb, &brsh, &bold);
		pDC->BeginPath();
		pDC->ArcTo(400 - d05, y + d05, 400 + d05, y - d05, 700, y - 400, 300, y + 400);
		pDC->LineTo(400, y);
		pDC->EndPath();
		pDC->StrokeAndFillPath();

		lb.lbColor = RGB(230, 230, 230);
		SelectBrush(pDC, &lb, &brsh, &bold);
		pDC->BeginPath();
		pDC->ArcTo(400 - d05, y + d05, 400 + d05, y - d05, 300, y + 400, 100, y);
		pDC->EndPath();
		pDC->StrokeAndFillPath();

		y += 300;
		SelectPen(pDC, PS_SOLID, 2, RGB(200, 200, 200), &pen, &pold);
		pDC->MoveTo(1000, y);
		pDC->LineTo(1000, y - 600);
		pDC->LineTo(1700, y - 600);
		for(i = 1050; i <= 1700; i += 50)
		{
			pDC->MoveTo(i, y - 600);
			pDC->LineTo(i, y - 610);
		}
		for(i = y - 600 + 50; i <= y; i += 50)
		{
			pDC->MoveTo(990, i);
			pDC->LineTo(1000, i);
		}
		pDC->FillRect(CRect(1050, y - 100, 1150, y - 600), &brshbm1);
		pDC->FillRect(CRect(1200, y - 200, 1300, y - 600), &brshbm4);
		pDC->FillRect(CRect(1350, y - 500, 1450, y - 600), &brshbm2);
		pDC->FillRect(CRect(1500, y - 50, 1600, y - 600), &brshbm6);
	}
	else if(page == 1)
	{
		lf.lfHeight = 50;
		lf.lfWeight = FW_NORMAL;
		SelectFont(pDC, &lf, &fnt, fold);
		pDC->TextOut(100, -500, "The content of the second page...");
	}

	SelectFont(pDC, NULL, &fnt, fold);
	SelectPen(pDC, 0, -1, 0, &pen, &pold);
	SelectBrush(pDC, NULL, &brsh, &bold);
	SelectBrush(pDC, NULL, &brshbm1, &bold);
	SelectBrush(pDC, NULL, &brshbm2, &bold);
	SelectBrush(pDC, NULL, &brshbm3, &bold);
	SelectBrush(pDC, NULL, &brshbm4, &bold);
	SelectBrush(pDC, NULL, &brshbm5, &bold);
	SelectBrush(pDC, NULL, &brshbm6, &bold);

}

CFont* CChildView::SelectFont(CDC* pDC, LOGFONT* lf, CFont* f, CFont* fold)
{
	if(fold != NULL)
		pDC->SelectObject(fold);
	if(f->m_hObject)
		f->DeleteObject();
	if(lf != NULL)
		f->CreateFontIndirect(lf);
	if((pDC == NULL) || (f->m_hObject == 0))
		return NULL;
	return pDC->SelectObject(f);
}

BYTE* CChildView::GetDIBImage(const TCHAR* bm)
{
	BYTE* pDib = NULL;
	CString s;
	s.Format(_T("%s%s"), DATADIR, bm);
	HBITMAP hBmp = (HBITMAP)LoadImage(0, s, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE | LR_CREATEDIBSECTION);
	if(hBmp != NULL)
	{
		BITMAPINFO bmi;
		memset(&bmi, 0, sizeof(BITMAPINFO));
		bmi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
		HDC hDC = ::GetDC(0);
		BYTE* pBits = NULL;
		long szb = 0;
		if(GetDIBits(hDC, hBmp, 0, 0, NULL, &bmi, DIB_RGB_COLORS) > 0)
		{
			bmi.bmiHeader.biBitCount = 24;
			bmi.bmiHeader.biCompression = BI_RGB;
			szb = bmi.bmiHeader.biHeight * (((bmi.bmiHeader.biWidth * 3) / 4) * 4 + (((bmi.bmiHeader.biWidth * 3) % 4) ? 4 : 0));
			pBits = new BYTE[szb];
		}
		if(pBits != NULL)
		{
			GetDIBits(hDC, hBmp, 0, bmi.bmiHeader.biHeight, pBits, &bmi, DIB_RGB_COLORS);
			::ReleaseDC(0, hDC);
			pDib = new BYTE[40 + szb];
		}
		if(pDib != NULL)
		{
			memcpy(pDib, &bmi, 40);
			memcpy(pDib + 40, pBits, szb);
		}
		delete[] pBits;

		DeleteObject(hBmp);
	}
	else
	{
		TRACE(_T("Error loading image %s\n"), s);
		return NULL;
	}
	return pDib;
}

long CChildView::SelectBrush(CDC* pDC, LOGBRUSH* lb, CBrush* b, CBrush** bold, const TCHAR* bm)
{
	if((pDC != NULL) && (*bold != NULL))
		pDC->SelectObject(*bold);
	if(b->m_hObject)
		b->DeleteObject();
	BYTE* pDib = NULL;
	if(bm != NULL)
		pDib = GetDIBImage(bm);
	if(pDib)
	{
		b->CreateDIBPatternBrush((CONST VOID*)pDib, DIB_RGB_COLORS);
		delete[] pDib;
	}
	else if(lb != NULL)
		b->CreateBrushIndirect(lb);

	if((pDC == NULL) || (b->m_hObject == 0))
		return 0;
	CBrush* _b = pDC->SelectObject(b);
	if(*bold == NULL)
		*bold = _b;
	return 1;
}

long CChildView::SelectPen(CDC* pDC, long style, long width, COLORREF col, CPen* p, CPen** pold, LOGBRUSH* lb, const TCHAR* bm)
{
	if((pDC != NULL) && (*pold != NULL))
		pDC->SelectObject(*pold);
	if(p->m_hObject)
		p->DeleteObject();
	BYTE* pDib = NULL;
	if((lb != NULL) && (bm != NULL))
	{
		lb->lbColor = col;
		lb->lbStyle = BS_DIBPATTERNPT;
		pDib = GetDIBImage(bm);
		lb->lbHatch = (long)pDib;
	}
	if(width >= 0)
	{
		if(lb != NULL)
			p->CreatePen(style, width, lb);
		else
			p->CreatePen(style, width, col);
	}
	if(pDib)
		delete[] pDib;
	if((pDC == NULL) || (p->m_hObject == 0))
		return 0;
	CPen* _p = pDC->SelectObject(p);
	if(*pold == NULL)
		*pold = _p;
	return 1;
}

void CChildView::OnSize(UINT nType, int cx, int cy) 
{
	CWnd::OnSize(nType, cx, cy);
	UpdScrollInfo();
}

void CChildView::UpdScrollInfo()
{
	CRect r;
	GetClientRect(r);

	m_offs.cx = 0;
	m_offs.cy = 0;
	long cx = m_szPageDP.cx * m_zoom / 100;
	long cy = ((m_szPageDP.cy * m_pages) + ((m_pages - 1) * 10)) * m_zoom / 100;
	if(r.Width() > cx)
		m_offs.cx = (r.Width() - cx) / 2;
	if(r.Height() > cy)
		m_offs.cy = (r.Height() - cy) / 2;

	SCROLLINFO si;
	si.cbSize = sizeof(SCROLLINFO);
	si.fMask = SIF_RANGE | SIF_PAGE | SIF_POS;
	for(int i = 0; i < 2; i++)
	{
		long sb = (i == 0 ? SB_HORZ : SB_VERT);
		GetScrollInfo(sb, &si);
		long p = si.nPos;
		long m = si.nMax;

		si.nPage = 1;
		si.nMin = 0;
		double z = m_zoom / 100.0;
		long d = (i == 0 ? cx : cy) - (i == 0 ? r.Width() : r.Height());
		long t = (long)((i == 0 ? m_scrd.cx : m_scrd.cy) * z);
		si.nMax = d / t + ((d % t > 0) ? 1 : 0);
		if(si.nMax < 0)
			si.nMax = 0;

		long pn = si.nPos;
		si.nPos = 0;
		SetScrollInfo(sb, &si);
		if(si.nMax != m)
		{
			pn = (long)(si.nMax * (p / (double)m));
			SetScrollInfo(sb, &si);
		}
		UpdScrollPos(sb, SB_THUMBTRACK, pn);
	}
}

void CChildView::UpdScrollPos(long sb, long code, long pos)
{
	SCROLLINFO si;
	si.cbSize = sizeof(SCROLLINFO);
	si.fMask = SIF_POS | SIF_RANGE;
	GetScrollInfo(sb, &si);


	switch(code)
	{
	case SB_THUMBTRACK:
		break;
	case SB_LINEUP:
		if(sb == SB_HORZ)
			pos = si.nPos - 1;
		else
			pos = si.nPos + 1;
		break;
	case SB_LINEDOWN:
		if(sb == SB_HORZ)
			pos = si.nPos + 1;
		else
			pos = si.nPos - 1;
		break;
	default:
		return;
	}
	if(pos < si.nMin)
		pos = si.nMin;
	if(pos > si.nMax)
		pos = si.nMax;

	long sdx = m_scrd.cx * m_zoom / 100;
	long sdy = m_scrd.cy * m_zoom / 100;
	if(si.nPos != pos)
	{
		long dx = 0, dy = 0;
		if(sb == SB_HORZ)
			dx = (si.nPos - pos) * sdx;
		else
			dy = (si.nPos - pos) * sdy;

		si.fMask = SIF_POS;
		si.nPos = pos;
		SetScrollInfo(sb, &si);

		CRect rc;
		GetClientRect(rc);
		ScrollWindowEx(dx, dy, NULL, rc, NULL, NULL, SW_ERASE | SW_INVALIDATE);
		m_offs.cx += dx;
		m_offs.cy += dy;
	}
}


void CChildView::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{
	UpdScrollPos(SB_VERT, nSBCode, nPos);
}

void CChildView::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{
	UpdScrollPos(SB_HORZ, nSBCode, nPos);
}

BOOL CChildView::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo) 
{
	switch(nCode)
	{
	case CN_COMMAND:
		{
			long z = 0;
			switch(nID)
			{
			case ID_ZOOM200: z = 200; break;
			case ID_ZOOM150: z = 150; break;
			case ID_ZOOM100: z = 100; break;
			case ID_ZOOM75: z = 75; break;
			case ID_ZOOM50: z = 50; break;
			case ID_ZOOM25: z = 25; break;
			case ID_EXPPDF: Export(); break;
			case ID_FILE_PRINT: OnPrint(); break;
			case ID_PDFOPT:
				{
					CDlgExp dlg(m_pdfopts, m_expfile, this);
					if(dlg.DoModal() == IDOK)
					{
						m_pdfopts = dlg.GetPDFOptions();
						m_expfile = dlg.GetExpFile();
					}
				}
				break;
			}
			if(z > 0)
			{
				m_zoom = z;
				UpdScrollInfo();
				Invalidate();
			}
		}
		break;
	case CN_UPDATE_COMMAND_UI:
		if(!pExtra)
			return FALSE;
		((CCmdUI*)pExtra)->Enable(TRUE);
		return TRUE;
		break;
	}

	
	return CWnd::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
}
